# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.system_object as system_binding
from cybox.common import PlatformSpecification, ObjectProperties, String, UnsignedLong, Date, DateTime, Duration, EnvironmentVariableList
from cybox.common.properties import Time
from cybox.objects.address_object import Address


class DHCPServerList(entities.EntityList):
    _binding_class = system_binding.DHCPServerListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    dhcp_server_address = fields.TypedField("DHCP_Server_Address", Address, multiple=True)


class IPGatewayList(entities.EntityList):
    _binding_class = system_binding.IPGatewayListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    ip_gateway_address = fields.TypedField("IP_Gateway_Address", Address, multiple=True)


class IPInfo(entities.Entity):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    _binding = system_binding
    _binding_class = system_binding.IPInfoType

    ip_address = fields.TypedField("IP_Address", Address)
    subnet_mask = fields.TypedField("Subnet_Mask", Address)


class IPInfoList(entities.EntityList):
    _binding_class = system_binding.IPInfoListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    ip_info_list = fields.TypedField("IP_Info_List", IPInfo, multiple=True)


class OS(entities.Entity):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    _binding = system_binding
    _binding_class = system_binding.OSType

    bitness = fields.TypedField("Bitness", String)
    build_number = fields.TypedField("Build_Number", String)
    environment_variable_list = fields.TypedField("Environment_Variable_List", EnvironmentVariableList)
    install_date = fields.TypedField("Install_Date", Date)
    patch_level = fields.TypedField("Patch_Level", String)
    platform = fields.TypedField("Platform", PlatformSpecification)


class BIOSInfo(entities.Entity):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    _binding = system_binding
    _binding_class = system_binding.BIOSInfoType

    bios_date = fields.TypedField("BIOS_Date", Date)
    bios_version = fields.TypedField("BIOS_Version", String)
    bios_manufacturer = fields.TypedField("BIOS_Manufacturer", String)
    bios_release_date = fields.TypedField("BIOS_Release_Date", Date)
    bios_serial_number = fields.TypedField("BIOS_Serial_Number", String)


class NetworkInterface(entities.Entity):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    _binding = system_binding
    _binding_class = system_binding.NetworkInterfaceType

    adapter = fields.TypedField("Adapter", String)
    description = fields.TypedField("Description", String)
    dhcp_lease_expires = fields.TypedField("DHCP_Lease_Expires", DateTime)
    dhcp_lease_obtained = fields.TypedField("DHCP_Lease_Obtained", DateTime)
    dhcp_server_list = fields.TypedField("DHCP_Server_List", DHCPServerList)
    ip_gateway_list = fields.TypedField("IP_Gateway_List", IPGatewayList)
    ip_list = fields.TypedField("IP_List", IPInfoList)
    mac = fields.TypedField("MAC", String)


class NetworkInterfaceList(entities.EntityList):
    _binding_class = system_binding.NetworkInterfaceListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    network_interface = fields.TypedField("Network_Interface", NetworkInterface, multiple=True)


class System(ObjectProperties):
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/system-2"
    _XSI_NS = "SystemObj"
    _XSI_TYPE = "SystemObjectType"
    _binding = system_binding
    _binding_class = system_binding.SystemObjectType

    available_physical_memory = fields.TypedField("Available_Physical_Memory", UnsignedLong)
    bios_info = fields.TypedField("BIOS_Info", BIOSInfo)
    date = fields.TypedField("Date", Date)
    hostname = fields.TypedField("Hostname", String)
    local_time = fields.TypedField("Local_Time", Time)
    network_interface_list = fields.TypedField("Network_Interface_List", NetworkInterfaceList)
    os = fields.TypedField("OS", OS)
    processor = fields.TypedField("Processor", String)
    # TODO
    # processor_architecture = fields.TypedField("Processor_Architecture", ProcessorArch)
    system_time = fields.TypedField("System_Time", Time)
    timezone_dst = fields.TypedField("Timezone_DST", String)
    timezone_standard = fields.TypedField("Timezone_Standard", String)
    total_physical_memory = fields.TypedField("Total_Physical_Memory", UnsignedLong)
    uptime = fields.TypedField("Uptime", Duration)
    username = fields.TypedField("Username", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.win_driver_object import WinDriver

from cybox.compat import long
from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestWinDriver(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsDriverObjectType"
    klass = WinDriver

    _full_dict = {
        'driver_init': 123,
        'driver_name': "A driver name",
        'driver_object_address': "abcde12345",
        'driver_start_io': "abcce4321",
        'driver_unload': "ab3234dec",
        'image_base': "12345abc",
        'image_size': "12ff",
        'irp_mj_cleanup': long(1),
        'irp_mj_close': long(2),
        'irp_mj_create': long(3),
        'irp_mj_create_mailslot': long(4),
        'irp_mj_create_named_pipe': long(5),
        'irp_mj_device_change': long(6),
        'irp_mj_device_control': long(7),
        'irp_mj_directory_control': long(8),
        'irp_mj_file_system_control': long(9),
        'irp_mj_flush_buffers': long(11),
        'irp_mj_internal_device_control': long(12),
        'irp_mj_lock_control': long(13),
        'irp_mj_pnp': long(14),
        'irp_mj_power': long(15),
        'irp_mj_query_ea': long(16),
        'irp_mj_query_information': long(17),
        'irp_mj_query_quota': long(22),
        'irp_mj_query_security': long(23),
        'irp_mj_query_volume_information': long(24),
        'irp_mj_read': long(25),
        'irp_mj_set_ea': long(26),
        'irp_mj_set_information': long(27),
        'irp_mj_set_quota': long(33),
        'irp_mj_set_security': long(34),
        'irp_mj_set_volume_information': long(35),
        'irp_mj_shutdown': long(36),
        'irp_mj_system_control': long(37),
        'irp_mj_write': long(38),
        #TODO: add 'device_object_list'
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import system_object
from . import win_handle_object


class GlobalFlagListType(GeneratedsSuper):
    """The GlobalFlagListType type is a listing of all Windows global
    flags."""

    subclass = None
    superclass = None
    def __init__(self, Global_Flag=None):
        if Global_Flag is None:
            self.Global_Flag = []
        else:
            self.Global_Flag = Global_Flag
    def factory(*args_, **kwargs_):
        if GlobalFlagListType.subclass:
            return GlobalFlagListType.subclass(*args_, **kwargs_)
        else:
            return GlobalFlagListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Global_Flag(self): return self.Global_Flag
    def set_Global_Flag(self, Global_Flag): self.Global_Flag = Global_Flag
    def add_Global_Flag(self, value): self.Global_Flag.append(value)
    def insert_Global_Flag(self, index, value): self.Global_Flag[index] = value
    def hasContent_(self):
        if (
            self.Global_Flag
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinSystemObj:', name_='GlobalFlagListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='GlobalFlagListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinSystemObj:', name_='GlobalFlagListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinSystemObj:', name_='GlobalFlagListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Global_Flag_ in self.Global_Flag:
            Global_Flag_.export(lwrite, level, 'WinSystemObj:', name_='Global_Flag', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Global_Flag':
            obj_ = GlobalFlagType.factory()
            obj_.build(child_)
            self.Global_Flag.append(obj_)
# end class GlobalFlagListType

class GlobalFlagType(GeneratedsSuper):
    """The GlobalFlagType type is intended to characterize Windows global
    flags."""

    subclass = None
    superclass = None
    def __init__(self, Abbreviation=None, Destination=None, Hexadecimal_Value=None, Symbolic_Name=None):
        self.Abbreviation = Abbreviation
        self.Destination = Destination
        self.Hexadecimal_Value = Hexadecimal_Value
        self.Symbolic_Name = Symbolic_Name
    def factory(*args_, **kwargs_):
        if GlobalFlagType.subclass:
            return GlobalFlagType.subclass(*args_, **kwargs_)
        else:
            return GlobalFlagType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Abbreviation(self): return self.Abbreviation
    def set_Abbreviation(self, Abbreviation): self.Abbreviation = Abbreviation
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Destination(self): return self.Destination
    def set_Destination(self, Destination): self.Destination = Destination
    def get_Hexadecimal_Value(self): return self.Hexadecimal_Value
    def set_Hexadecimal_Value(self, Hexadecimal_Value): self.Hexadecimal_Value = Hexadecimal_Value
    def validate_HexBinaryObjectPropertyType(self, value):
        # Validate type cybox_common.HexBinaryObjectPropertyType, a restriction on None.
        pass
    def get_Symbolic_Name(self): return self.Symbolic_Name
    def set_Symbolic_Name(self, Symbolic_Name): self.Symbolic_Name = Symbolic_Name
    def hasContent_(self):
        if (
            self.Abbreviation is not None or
            self.Destination is not None or
            self.Hexadecimal_Value is not None or
            self.Symbolic_Name is not None
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinSystemObj:', name_='GlobalFlagType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='GlobalFlagType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinSystemObj:', name_='GlobalFlagType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='WinSystemObj:', name_='GlobalFlagType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Abbreviation is not None:
            self.Abbreviation.export(lwrite, level, 'WinSystemObj:', name_='Abbreviation', pretty_print=pretty_print)
        if self.Destination is not None:
            self.Destination.export(lwrite, level, 'WinSystemObj:', name_='Destination', pretty_print=pretty_print)
        if self.Hexadecimal_Value is not None:
            self.Hexadecimal_Value.export(lwrite, level, 'WinSystemObj:', name_='Hexadecimal_Value', pretty_print=pretty_print)
        if self.Symbolic_Name is not None:
            self.Symbolic_Name.export(lwrite, level, 'WinSystemObj:', name_='Symbolic_Name', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Abbreviation':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Abbreviation(obj_)
        elif nodeName_ == 'Destination':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Destination(obj_)
        elif nodeName_ == 'Hexadecimal_Value':
            obj_ = cybox_common.HexBinaryObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Hexadecimal_Value(obj_)
        elif nodeName_ == 'Symbolic_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Symbolic_Name(obj_)
# end class GlobalFlagType

class WindowsSystemObjectType(system_object.SystemObjectType):
    """The WindowsSystemObjectType type is intended to characterize Windows
    systems."""

    subclass = None
    superclass = system_object.SystemObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Available_Physical_Memory=None, BIOS_Info=None, Date=None, Hostname=None, Local_Time=None, Network_Interface_List=None, OS=None, Processor=None, Processor_Architecture=None, System_Time=None, Timezone_DST=None, Timezone_Standard=None, Total_Physical_Memory=None, Uptime=None, Username=None, Domain=None, Global_Flag_List=None, NetBIOS_Name=None, Open_Handle_List=None, Product_ID=None, Product_Name=None, Registered_Organization=None, Registered_Owner=None, Windows_Directory=None, Windows_System_Directory=None, Windows_Temp_Directory=None):
        super(WindowsSystemObjectType, self).__init__(object_reference, Custom_Properties, xsi_type, Available_Physical_Memory, BIOS_Info, Date, Hostname, Local_Time, Network_Interface_List, OS, Processor, Processor_Architecture, System_Time, Timezone_DST, Timezone_Standard, Total_Physical_Memory, Uptime, Username, )
        if Domain is None:
            self.Domain = []
        else:
            self.Domain = Domain
        self.Global_Flag_List = Global_Flag_List
        self.NetBIOS_Name = NetBIOS_Name
        self.Open_Handle_List = Open_Handle_List
        self.Product_ID = Product_ID
        self.Product_Name = Product_Name
        self.Registered_Organization = Registered_Organization
        self.Registered_Owner = Registered_Owner
        self.Windows_Directory = Windows_Directory
        self.Windows_System_Directory = Windows_System_Directory
        self.Windows_Temp_Directory = Windows_Temp_Directory
    def factory(*args_, **kwargs_):
        if WindowsSystemObjectType.subclass:
            return WindowsSystemObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsSystemObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Domain(self): return self.Domain
    def set_Domain(self, Domain): self.Domain = Domain
    def add_Domain(self, value): self.Domain.append(value)
    def insert_Domain(self, index, value): self.Domain[index] = value
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Global_Flag_List(self): return self.Global_Flag_List
    def set_Global_Flag_List(self, Global_Flag_List): self.Global_Flag_List = Global_Flag_List
    def get_NetBIOS_Name(self): return self.NetBIOS_Name
    def set_NetBIOS_Name(self, NetBIOS_Name): self.NetBIOS_Name = NetBIOS_Name
    def get_Open_Handle_List(self): return self.Open_Handle_List
    def set_Open_Handle_List(self, Open_Handle_List): self.Open_Handle_List = Open_Handle_List
    def get_Product_ID(self): return self.Product_ID
    def set_Product_ID(self, Product_ID): self.Product_ID = Product_ID
    def get_Product_Name(self): return self.Product_Name
    def set_Product_Name(self, Product_Name): self.Product_Name = Product_Name
    def get_Registered_Organization(self): return self.Registered_Organization
    def set_Registered_Organization(self, Registered_Organization): self.Registered_Organization = Registered_Organization
    def get_Registered_Owner(self): return self.Registered_Owner
    def set_Registered_Owner(self, Registered_Owner): self.Registered_Owner = Registered_Owner
    def get_Windows_Directory(self): return self.Windows_Directory
    def set_Windows_Directory(self, Windows_Directory): self.Windows_Directory = Windows_Directory
    def get_Windows_System_Directory(self): return self.Windows_System_Directory
    def set_Windows_System_Directory(self, Windows_System_Directory): self.Windows_System_Directory = Windows_System_Directory
    def get_Windows_Temp_Directory(self): return self.Windows_Temp_Directory
    def set_Windows_Temp_Directory(self, Windows_Temp_Directory): self.Windows_Temp_Directory = Windows_Temp_Directory
    def hasContent_(self):
        if (
            self.Domain or
            self.Global_Flag_List is not None or
            self.NetBIOS_Name is not None or
            self.Open_Handle_List is not None or
            self.Product_ID is not None or
            self.Product_Name is not None or
            self.Registered_Organization is not None or
            self.Registered_Owner is not None or
            self.Windows_Directory is not None or
            self.Windows_System_Directory is not None or
            self.Windows_Temp_Directory is not None or
            super(WindowsSystemObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinSystemObj:', name_='WindowsSystemObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsSystemObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinSystemObj:', name_='WindowsSystemObjectType'):
        super(WindowsSystemObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsSystemObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinSystemObj:', name_='WindowsSystemObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsSystemObjectType, self).exportChildren(lwrite, level, 'WinSystemObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Domain_ in self.Domain:
            Domain_.export(lwrite, level, 'WinSystemObj:', name_='Domain', pretty_print=pretty_print)
        if self.Global_Flag_List is not None:
            self.Global_Flag_List.export(lwrite, level, 'WinSystemObj:', name_='Global_Flag_List', pretty_print=pretty_print)
        if self.NetBIOS_Name is not None:
            self.NetBIOS_Name.export(lwrite, level, 'WinSystemObj:', name_='NetBIOS_Name', pretty_print=pretty_print)
        if self.Open_Handle_List is not None:
            self.Open_Handle_List.export(lwrite, level, 'WinSystemObj:', name_='Open_Handle_List', pretty_print=pretty_print)
        if self.Product_ID is not None:
            self.Product_ID.export(lwrite, level, 'WinSystemObj:', name_='Product_ID', pretty_print=pretty_print)
        if self.Product_Name is not None:
            self.Product_Name.export(lwrite, level, 'WinSystemObj:', name_='Product_Name', pretty_print=pretty_print)
        if self.Registered_Organization is not None:
            self.Registered_Organization.export(lwrite, level, 'WinSystemObj:', name_='Registered_Organization', pretty_print=pretty_print)
        if self.Registered_Owner is not None:
            self.Registered_Owner.export(lwrite, level, 'WinSystemObj:', name_='Registered_Owner', pretty_print=pretty_print)
        if self.Windows_Directory is not None:
            self.Windows_Directory.export(lwrite, level, 'WinSystemObj:', name_='Windows_Directory', pretty_print=pretty_print)
        if self.Windows_System_Directory is not None:
            self.Windows_System_Directory.export(lwrite, level, 'WinSystemObj:', name_='Windows_System_Directory', pretty_print=pretty_print)
        if self.Windows_Temp_Directory is not None:
            self.Windows_Temp_Directory.export(lwrite, level, 'WinSystemObj:', name_='Windows_Temp_Directory', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsSystemObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Domain':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.Domain.append(obj_)
        elif nodeName_ == 'Global_Flag_List':
            obj_ = GlobalFlagListType.factory()
            obj_.build(child_)
            self.set_Global_Flag_List(obj_)
        elif nodeName_ == 'NetBIOS_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_NetBIOS_Name(obj_)
        elif nodeName_ == 'Open_Handle_List':
            obj_ = win_handle_object.WindowsHandleListType.factory()
            obj_.build(child_)
            self.set_Open_Handle_List(obj_)
        elif nodeName_ == 'Product_ID':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Product_ID(obj_)
        elif nodeName_ == 'Product_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Product_Name(obj_)
        elif nodeName_ == 'Registered_Organization':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Registered_Organization(obj_)
        elif nodeName_ == 'Registered_Owner':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Registered_Owner(obj_)
        elif nodeName_ == 'Windows_Directory':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Windows_Directory(obj_)
        elif nodeName_ == 'Windows_System_Directory':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Windows_System_Directory(obj_)
        elif nodeName_ == 'Windows_Temp_Directory':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Windows_Temp_Directory(obj_)
        super(WindowsSystemObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsSystemObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Open_Handle_List': win_handle_object.WindowsHandleListType,
    'DHCP_Lease_Expires': cybox_common.DateTimeObjectPropertyType,
    'Destination': cybox_common.StringObjectPropertyType,
    'Processor_Architecture': system_object.ProcessorArchType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Hexadecimal_Value': cybox_common.HexBinaryObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'BIOS_Date': cybox_common.DateObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Network_Interface_List': system_object.NetworkInterfaceListType,
    'NetBIOS_Name': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'IP_Gateway_List': system_object.IPGatewayListType,
    'Windows_System_Directory': cybox_common.StringObjectPropertyType,
    'Registered_Owner': cybox_common.StringObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Abbreviation': cybox_common.StringObjectPropertyType,
    'MAC': cybox_common.StringObjectPropertyType,
    'Object_Address': cybox_common.UnsignedLongObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'OS': system_object.OSType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Pointer_Count': cybox_common.UnsignedLongObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': system_object.SystemObjectType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Username': cybox_common.StringObjectPropertyType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Product_Name': cybox_common.StringObjectPropertyType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'BIOS_Info': system_object.BIOSInfoType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Registered_Organization': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Domain': cybox_common.StringObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'BIOS_Manufacturer': cybox_common.StringObjectPropertyType,
    'Reference_Description': cybox_common.StructuredTextType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Available_Physical_Memory': cybox_common.UnsignedLongObjectPropertyType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Uptime': cybox_common.DurationObjectPropertyType,
    'IP_List': system_object.IPInfoListType,
    'Timezone_DST': cybox_common.StringObjectPropertyType,
    'Hostname': cybox_common.StringObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Bitness': system_object.BitnessType,
    'BIOS_Version': cybox_common.StringObjectPropertyType,
    'Import': cybox_common.StringObjectPropertyType,
    'Access_Mask': cybox_common.UnsignedLongObjectPropertyType,
    'IP_Info': system_object.IPInfoType,
    'System_Time': cybox_common.TimeObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Build_Number': cybox_common.StringObjectPropertyType,
    'Adapter': cybox_common.StringObjectPropertyType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Patch_Level': cybox_common.StringObjectPropertyType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateObjectPropertyType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Windows_Directory': cybox_common.StringObjectPropertyType,
    'DHCP_Server_List': system_object.DHCPServerListType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Imports': cybox_common.ImportsType,
    'Install_Date': cybox_common.DateObjectPropertyType,
    'Total_Physical_Memory': cybox_common.UnsignedLongObjectPropertyType,
    'Library': cybox_common.LibraryType,
    'DHCP_Lease_Obtained': cybox_common.DateTimeObjectPropertyType,
    'References': cybox_common.ToolReferencesType,
    'Network_Interface': system_object.NetworkInterfaceType,
    'Windows_Handle': win_handle_object.WindowsHandleObjectType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Time': cybox_common.TimeType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Data_Size': cybox_common.DataSizeType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Timezone_Standard': cybox_common.StringObjectPropertyType,
    'Handle': win_handle_object.WindowsHandleObjectType,
    'Description': cybox_common.StringObjectPropertyType,
    'BIOS_Serial_Number': cybox_common.StringObjectPropertyType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Symbolic_Name': cybox_common.StringObjectPropertyType,
    'VLAN_Name': cybox_common.StringObjectPropertyType,
    'Local_Time': cybox_common.TimeObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Name': cybox_common.StringObjectPropertyType,
    'Processor': cybox_common.StringObjectPropertyType,
    'Address_Value': cybox_common.StringObjectPropertyType,
    'Environment_Variable_List': cybox_common.EnvironmentVariableListType,
    'VLAN_Num': cybox_common.IntegerObjectPropertyType,
    'Value': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'ID': cybox_common.UnsignedIntegerObjectPropertyType,
    'Windows_Temp_Directory': cybox_common.StringObjectPropertyType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'BIOS_Release_Date': cybox_common.DateObjectPropertyType,
    'Contributor': cybox_common.ContributorType,
    'Product_ID': cybox_common.StringObjectPropertyType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_System'
        rootClass = WindowsSystemObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_System'
        rootClass = WindowsSystemObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_System'
        rootClass = WindowsSystemObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="Windows_System",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsSystemObjectType",
    "GlobalFlagListType",
    "GlobalFlagType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.disk_partition_object as disk_partition_binding
from cybox.common import (DateTime, Integer, Name, ObjectProperties, String,
        UnsignedLong)


class DiskPartition(ObjectProperties):
    _binding = disk_partition_binding
    _binding_class = disk_partition_binding.DiskPartitionObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/disk-partition-2"
    _XSI_NS = "DiskPartitionObj"
    _XSI_TYPE = "DiskPartitionObjectType"

    created = fields.TypedField('Created', DateTime)
    device_name = fields.TypedField('Device_Name', Name)
    mount_point = fields.TypedField('Mount_Point', String)
    partition_id = fields.TypedField('Partition_ID', Integer)
    partition_length = fields.TypedField('Partition_Length', UnsignedLong)
    partition_offset = fields.TypedField('Partition_Offset', UnsignedLong)
    space_left = fields.TypedField('Space_Left', UnsignedLong)
    space_used = fields.TypedField('Space_Used', UnsignedLong)
    total_space = fields.TypedField('Total_Space', UnsignedLong)
    type_ = fields.TypedField('Type', String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import account_object


class PrivilegeListType(GeneratedsSuper):
    """The PrivilegeListType type specifies the list of privileges that the
    user account has."""

    subclass = None
    superclass = None
    def __init__(self, Privilege=None):
        if Privilege is None:
            self.Privilege = []
        else:
            self.Privilege = Privilege
    def factory(*args_, **kwargs_):
        if PrivilegeListType.subclass:
            return PrivilegeListType.subclass(*args_, **kwargs_)
        else:
            return PrivilegeListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Privilege(self): return self.Privilege
    def set_Privilege(self, Privilege): self.Privilege = Privilege
    def add_Privilege(self, value): self.Privilege.append(value)
    def insert_Privilege(self, index, value): self.Privilege[index] = value
    def hasContent_(self):
        if (
            self.Privilege
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='UserAccountObj:', name_='PrivilegeListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='PrivilegeListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='UserAccountObj:', name_='PrivilegeListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='UserAccountObj:', name_='PrivilegeListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Privilege_ in self.get_Privilege():
            Privilege_.export(lwrite, level, 'UserAccountObj:', name_='Privilege', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Privilege':
            type_name_ = child_.attrib.get(
                '{http://www.w3.org/2001/XMLSchema-instance}type')
            if type_name_ is None:
                type_name_ = child_.attrib.get('type')
            if type_name_ is not None:
                type_names_ = type_name_.split(':')
                if len(type_names_) == 1:
                    type_name_ = type_names_[0]
                else:
                    type_name_ = type_names_[1]
                class_ = globals()[type_name_]
                obj_ = class_.factory()
                obj_.build(child_)
            else:
                raise NotImplementedError(
                    'Class not implemented for <Privilege> element')
            self.Privilege.append(obj_)
# end class PrivilegeListType

class PrivilegeType(GeneratedsSuper):
    """The PrivilegeType type specifies a specific privilege that a user
    has. This is an abstract type since user privileges are
    operating-system specific, and is extended as needed in the
    derived CybOX object schemas."""

    subclass = None
    superclass = None
    def __init__(self):
        pass
    def factory(*args_, **kwargs_):
        if PrivilegeType.subclass:
            return PrivilegeType.subclass(*args_, **kwargs_)
        else:
            return PrivilegeType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def hasContent_(self):
        if (

            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='UserAccountObj:', name_='PrivilegeType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='PrivilegeType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='UserAccountObj:', name_='PrivilegeType'):
        lwrite(' xsi:type="%s%s"' % (namespace_, name_))
    def exportChildren(self, lwrite, level, namespace_='UserAccountObj:', name_='PrivilegeType', fromsubclass_=False, pretty_print=True):
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class PrivilegeType

class GroupListType(GeneratedsSuper):
    """The GroupListType type specifies the groups that the user account
    belongs to."""

    subclass = None
    superclass = None
    def __init__(self, Group=None):
        if Group is None:
            self.Group = []
        else:
            self.Group = Group
    def factory(*args_, **kwargs_):
        if GroupListType.subclass:
            return GroupListType.subclass(*args_, **kwargs_)
        else:
            return GroupListType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Group(self): return self.Group
    def set_Group(self, Group): self.Group = Group
    def add_Group(self, value): self.Group.append(value)
    def insert_Group(self, index, value): self.Group[index] = value
    def hasContent_(self):
        if (
            self.Group
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='UserAccountObj:', name_='GroupListType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='GroupListType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='UserAccountObj:', name_='GroupListType'):
        pass
    def exportChildren(self, lwrite, level, namespace_='UserAccountObj:', name_='GroupListType', fromsubclass_=False, pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        for Group_ in self.get_Group():
            Group_.export(lwrite, level, 'UserAccountObj:', name_='Group', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Group':
            type_name_ = child_.attrib.get(
                '{http://www.w3.org/2001/XMLSchema-instance}type')
            if type_name_ is None:
                type_name_ = child_.attrib.get('type')
            if type_name_ is not None:
                type_names_ = type_name_.split(':')
                if len(type_names_) == 1:
                    type_name_ = type_names_[0]
                else:
                    type_name_ = type_names_[1]
                class_ = globals()[type_name_]
                obj_ = class_.factory()
                obj_.build(child_)
            else:
                raise NotImplementedError(
                    'Class not implemented for <Group> element')
            self.Group.append(obj_)
# end class GroupListType

class GroupType(GeneratedsSuper):
    """The GroupType type specifies a group that a user account belongs to.
    This is an abstract type since group IDs are operating-system
    specific, and is extended as needed in the derived CybOX object
    schemas."""

    subclass = None
    superclass = None
    def __init__(self):
        pass
    def factory(*args_, **kwargs_):
        if GroupType.subclass:
            return GroupType.subclass(*args_, **kwargs_)
        else:
            return GroupType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def hasContent_(self):
        if (

            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='UserAccountObj:', name_='GroupType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='GroupType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='UserAccountObj:', name_='GroupType'):
        lwrite(' xsi:type="%s%s"' % (namespace_, name_))
    def exportChildren(self, lwrite, level, namespace_='UserAccountObj:', name_='GroupType', fromsubclass_=False, pretty_print=True):
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        pass
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class GroupType

class UserAccountObjectType(account_object.AccountObjectType):
    """The UserAccountObjectType type is intended to characterize generic
    user accounts.The password_required field specifies whether a
    password is required for this user account."""

    subclass = None
    superclass = account_object.AccountObjectType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, disabled=None, locked_out=None, Description=None, Domain=None, password_required=None, Full_Name=None, Group_List=None, Home_Directory=None, Last_Login=None, Privilege_List=None, Script_Path=None, Username=None, User_Password_Age=None):
        super(UserAccountObjectType, self).__init__(object_reference, Custom_Properties, disabled, locked_out, Description, Domain, )
        self.password_required = _cast(bool, password_required)
        self.Full_Name = Full_Name
        self.Group_List = Group_List
        self.Home_Directory = Home_Directory
        self.Last_Login = Last_Login
        self.Privilege_List = Privilege_List
        self.Script_Path = Script_Path
        self.Username = Username
        self.User_Password_Age = User_Password_Age
    def factory(*args_, **kwargs_):
        if UserAccountObjectType.subclass:
            return UserAccountObjectType.subclass(*args_, **kwargs_)
        else:
            return UserAccountObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Full_Name(self): return self.Full_Name
    def set_Full_Name(self, Full_Name): self.Full_Name = Full_Name
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_Group_List(self): return self.Group_List
    def set_Group_List(self, Group_List): self.Group_List = Group_List
    def get_Home_Directory(self): return self.Home_Directory
    def set_Home_Directory(self, Home_Directory): self.Home_Directory = Home_Directory
    def get_Last_Login(self): return self.Last_Login
    def set_Last_Login(self, Last_Login): self.Last_Login = Last_Login
    def validate_DateTimeObjectPropertyType(self, value):
        # Validate type cybox_common.DateTimeObjectPropertyType, a restriction on None.
        pass
    def get_Privilege_List(self): return self.Privilege_List
    def set_Privilege_List(self, Privilege_List): self.Privilege_List = Privilege_List
    def get_Script_Path(self): return self.Script_Path
    def set_Script_Path(self, Script_Path): self.Script_Path = Script_Path
    def get_Username(self): return self.Username
    def set_Username(self, Username): self.Username = Username
    def get_User_Password_Age(self): return self.User_Password_Age
    def set_User_Password_Age(self, User_Password_Age): self.User_Password_Age = User_Password_Age
    def validate_DurationObjectPropertyType(self, value):
        # Validate type cybox_common.DurationObjectPropertyType, a restriction on None.
        pass
    def get_password_required(self): return self.password_required
    def set_password_required(self, password_required): self.password_required = password_required
    def hasContent_(self):
        if (
            self.Full_Name is not None or
            self.Group_List is not None or
            self.Home_Directory is not None or
            self.Last_Login is not None or
            self.Privilege_List is not None or
            self.Script_Path is not None or
            self.Username is not None or
            self.User_Password_Age is not None or
            super(UserAccountObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='UserAccountObj:', name_='UserAccountObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='UserAccountObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='UserAccountObj:', name_='UserAccountObjectType'):
        super(UserAccountObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='UserAccountObjectType')
        if self.password_required is not None:

            lwrite(' password_required="%s"' % self.gds_format_boolean(self.password_required, input_name='password_required'))
    def exportChildren(self, lwrite, level, namespace_='UserAccountObj:', name_='UserAccountObjectType', fromsubclass_=False, pretty_print=True):
        super(UserAccountObjectType, self).exportChildren(lwrite, level, 'UserAccountObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Full_Name is not None:
            self.Full_Name.export(lwrite, level, 'UserAccountObj:', name_='Full_Name', pretty_print=pretty_print)
        if self.Group_List is not None:
            self.Group_List.export(lwrite, level, 'UserAccountObj:', name_='Group_List', pretty_print=pretty_print)
        if self.Home_Directory is not None:
            self.Home_Directory.export(lwrite, level, 'UserAccountObj:', name_='Home_Directory', pretty_print=pretty_print)
        if self.Last_Login is not None:
            self.Last_Login.export(lwrite, level, 'UserAccountObj:', name_='Last_Login', pretty_print=pretty_print)
        if self.Privilege_List is not None:
            self.Privilege_List.export(lwrite, level, 'UserAccountObj:', name_='Privilege_List', pretty_print=pretty_print)
        if self.Script_Path is not None:
            self.Script_Path.export(lwrite, level, 'UserAccountObj:', name_='Script_Path', pretty_print=pretty_print)
        if self.Username is not None:
            self.Username.export(lwrite, level, 'UserAccountObj:', name_='Username', pretty_print=pretty_print)
        if self.User_Password_Age is not None:
            self.User_Password_Age.export(lwrite, level, 'UserAccountObj:', name_='User_Password_Age', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        value = find_attr_value_('password_required', node)
        if value is not None:

            if value in ('true', '1'):
                self.password_required = True
            elif value in ('false', '0'):
                self.password_required = False
            else:
                raise_parse_error(node, 'Bad boolean attribute')
        super(UserAccountObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Full_Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Full_Name(obj_)
        elif nodeName_ == 'Group_List':
            obj_ = GroupListType.factory()
            obj_.build(child_)
            self.set_Group_List(obj_)
        elif nodeName_ == 'Home_Directory':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Home_Directory(obj_)
        elif nodeName_ == 'Last_Login':
            obj_ = cybox_common.DateTimeObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Last_Login(obj_)
        elif nodeName_ == 'Privilege_List':
            obj_ = PrivilegeListType.factory()
            obj_.build(child_)
            self.set_Privilege_List(obj_)
        elif nodeName_ == 'Script_Path':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Script_Path(obj_)
        elif nodeName_ == 'Username':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Username(obj_)
        elif nodeName_ == 'User_Password_Age':
            obj_ = cybox_common.DurationObjectPropertyType.factory()
            obj_.build(child_)
            self.set_User_Password_Age(obj_)
        super(UserAccountObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class UserAccountObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'User_Password_Age': cybox_common.DurationObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Home_Directory': cybox_common.StringObjectPropertyType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Account': account_object.AccountObjectType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Compiler': cybox_common.CompilerType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': cybox_common.ControlledVocabularyStringType,
    'Compilers': cybox_common.CompilersType,
    'Username': cybox_common.StringObjectPropertyType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Tool': cybox_common.ToolInformationType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Language': cybox_common.StringObjectPropertyType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'Domain': cybox_common.StringObjectPropertyType,
    'Contributors': cybox_common.PersonnelType,
    'Reference_Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Name': cybox_common.StringObjectPropertyType,
    'Import': cybox_common.StringObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Full_Name': cybox_common.StringObjectPropertyType,
    'Error': cybox_common.ErrorType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Script_Path': cybox_common.StringObjectPropertyType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Description': cybox_common.StringObjectPropertyType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Last_Login': cybox_common.DateTimeObjectPropertyType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'User_Account'
        rootClass = UserAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_=rootTag,
#        namespacedef_='',
#        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'User_Account'
        rootClass = UserAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'User_Account'
        rootClass = UserAccountObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
#    sys.stdout.write('<?xml version="1.0" ?>\n')
#    rootObj.export(sys.stdout.write, 0, name_="User_Account",
#        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "UserAccountObjectType",
    "PrivilegeListType",
    "PrivilegeType",
    "GroupListType",
    "GroupType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox import entities
from mixbox.vendor.six import u

from cybox.bindings import cybox_common as common_binding
from cybox.common import HashName, VocabString
from cybox.common.vocabs import VocabField
from cybox.common.vocabs import HashName as HashNameVocab  # Backwards compatibility

import cybox.test


class MultipleHash(entities.Entity):
    """Fake entity class to test multiple on VocabField."""
    _binding = common_binding
    _binding_class = common_binding.HashType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    type_ = VocabField("Type", HashName, multiple=True)


class TestMultipleVocabField(unittest.TestCase):
    def test_multiple_str(self):
        mh = MultipleHash()
        mh.type_  = [HashNameVocab.TERM_MD5, HashNameVocab.TERM_SHA1]
        self.assertEqual(len(mh.type_), 2)

    def test_multiple_mixed(self):
        mh = MultipleHash()
        mh.type_ = [HashNameVocab.TERM_MD5, VocabString("Foobar")]
        self.assertEqual(len(mh.type_), 2)

    def test_single_value(self):
        mh = MultipleHash()
        mh.type_ = HashNameVocab.TERM_MD5
        self.assertEqual(len(mh.type_), 1)

    def test_nonvalid_item(self):
        mh = MultipleHash()

        self.assertRaises(
            ValueError,
            setattr,
            mh,
            'type_',
            [HashNameVocab.TERM_MD5, "THIS SHOULD FAIL"]
        )

    def test_from_empty_dict(self):
        mh = MultipleHash.from_dict({})
        d = mh.to_dict()
        self.assertEqual(len(d), 0)


class TestVocabString(unittest.TestCase):

    def test_plain(self):
        a = VocabString("test_value")
        self.assertTrue(a.is_plain())

    def test_is_plain_hashname(self):
        md5 = HashName("MD5")
        self.assertTrue(md5.is_plain())

    def test_is_plain_hashname(self):
        md5 = HashName("MD5")
        md5.xsi_type = "Some Other xsi:type"
        self.assertFalse(md5.is_plain())

    def test_round_trip(self):
        vocab_dict = {
                        'value': "test_value",
                        'vocab_name': "test_a",
                        'vocab_reference': "test_b",

                        'condition': "test_d",
                        # Leave out apply_condition since value is not a list.
                        'bit_mask': "test_1",
                        'pattern_type': "test_e",
                        'regex_syntax': "test_f",
                        'has_changed': "test_j",
                        'trend': "test_k",
                     }

        vocab_dict2 = cybox.test.round_trip_dict(VocabString, vocab_dict)
        cybox.test.assert_equal_ignore(vocab_dict, vocab_dict2, ['xsi:type'])

    # https://github.com/CybOXProject/python-cybox/issues/158
    def test_xsi_type_unicode(self):
        string = u("test\u2010value")
        vocab_dict = {
                        'value': string,
                        'condition': u("Equals"),
                        'xsi:type': u("some_xsi_type"),
                     }

        vocab_dict2 = cybox.test.round_trip_dict(VocabString, vocab_dict)
        self.assertEqual(vocab_dict, vocab_dict2)
        xml = VocabString.from_dict(vocab_dict).to_xml()
        string_utf8 = string.encode("utf-8")
        self.assertTrue(string_utf8 in xml)

    def test_round_trip_list(self):
        vocab_dict = {
                        'value': ['Value1', 'Value2', 'Value3'],
                        'condition': "Equals",
                        'apply_condition': "ALL",
                        'vocab_name': "Test",
                     }

        vocab_dict2 = cybox.test.round_trip_dict(VocabString, vocab_dict)
        cybox.test.assert_equal_ignore(vocab_dict, vocab_dict2, ['xsi:type'])

    def test_add_bad_value(self):
        from cybox.common import Hash

        h = Hash()
        self.assertRaises(
            ValueError,
            setattr,
            h,
            'type_',
            "BAD VALUE"
        )

    def test_add_vocabstring(self):
        from cybox.common import Hash
        from cybox.common.vocabs import ActionName

        action = ActionName(ActionName.TERM_ADD_USER)
        h = Hash()
        h.type_ = action

        self.assertEqual(action, h.type_)

    def test_to_dict(self):
        from cybox.common.vocabs import ActionName
        d = ActionName(ActionName.TERM_ADD_USER).to_dict()
        if 'xsi:type' in d:
            self.assertEqual(d['xsi:type'], ActionName._XSI_TYPE)
        if 'value' in d:
            self.assertEqual(d['value'], ActionName.TERM_ADD_USER)
        else:
            self.assertEqual(d, ActionName.TERM_ADD_USER)


class HashNameTests(unittest.TestCase):

    def test_hash_name_vocabulary(self):
        # Test for using the @register_vocab decorator.
        self.assertEqual(8, len(HashNameVocab._ALLOWED_VALUES))


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.win_kernel_object import WinKernel

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestWinKernel(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsKernelObjectType"
    klass = WinKernel

    _full_dict = {
        'idt': [
            {
                'type_attr': "897987987d9c87de",
                'offset_high': "78675b67a576e",
                'offset_low': "3543c5d3e43a5",
                'offset_middle': "abcde142897",
                'selector': "edcba1412"
            },
            {
                'type_attr': "897987987d9c87de11111",
                'offset_high': "78675b67a576e11111",
                'offset_low': "3543c5d3e43a511111",
                'offset_middle': "abcde14289711111",
                'selector': "edcba141211111"
            }
        ],
        'ssdt': [
            {
                'hooked': False,
                'service_table_base': "a86dc86b6e",
                'service_counter_table_base': "098b89e098dc809a",
                'number_of_services': 2,
                'argument_table_base': "809a09ad890c89ee14124"
            }, 
            {
                'hooked': True,
                'service_table_base': "a86dc86b6e11111",
                'service_counter_table_base': "098b89e098dc809a11111",
                'number_of_services': 3,
                'argument_table_base': "809a09ad890c89ee1412411111"
            }
        ],
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python

# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

'''
CybOX Common Indicator helper Demo

Demonstrates the use of the Cybox Common Indicator helper.
Creates a CybOX Observables document containing a
'''

import sys
from pprint import pprint

from cybox import helper
from cybox.core import Observables


def main():
    '''Build a CybOX Observables document and write it to stdout'''
    domain = helper.create_domain_name_observable('www.example.com')
    url = helper.create_url_observable('http://www.example.com')
    ipv4 = helper.create_ipv4_observable('127.0.0.1')
    email = helper.create_email_address_observable('cybox@mitre.org')
    file_ = helper.create_file_hash_observable('foo.bar',
                                            '94f93e00fd122466d68a6ae3b8c7f908')

    observables_doc = Observables([
                                    domain,
                                    ipv4,
                                    url,
                                    email,
                                    file_,
                                  ])
    print(observables_doc.to_xml(encoding=None))

    pprint(observables_doc.to_dict())

if __name__ == "__main__":
    main()
    sys.exit()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.


import unittest

from cybox.utils.normalize import normalize_object_properties
from cybox.objects.file_object import File
from cybox.objects.win_registry_key_object import WinRegistryKey, RegistryValues, RegistryValue
from cybox.objects.process_object import Process, ImageInfo
from cybox.objects.mutex_object import Mutex

class TestNormalization(unittest.TestCase):

    def test_file_path(self):
        file_path_string = "%WinDir%\abcd.dll"
        normalized_file_path_string = "CSIDL_WINDOWS\abcd.dll"

        file_obj = File()
        file_obj.file_path = file_path_string

        normalize_object_properties(file_obj)

        self.assertEqual(file_obj.file_path.value, normalized_file_path_string)

    def test_registry_value_data(self):
        file_path_string = "C:"
        normalized_file_path_string = "%SystemDrive%"

        registry_key_obj = WinRegistryKey()
        registry_key_obj.values = RegistryValues()
        registry_value = RegistryValue()
        registry_value.data = file_path_string
        registry_key_obj.values.append(registry_value)

        normalize_object_properties(registry_key_obj)

        self.assertEqual(registry_key_obj.values[0].data.value, normalized_file_path_string)

    def test_registry_hive(self):
        hive_string = "HKLM"
        normalized_hive_string = "HKEY_LOCAL_MACHINE"

        registry_key_obj = WinRegistryKey()
        registry_key_obj.hive = hive_string

        normalize_object_properties(registry_key_obj)

        self.assertEqual(registry_key_obj.hive.value, normalized_hive_string)
        
    def test_process_image_info_path(self):
        file_path_string = "C:\Windows\System32\abcd.dll"
        normalized_file_path_string = "CSIDL_SYSTEM\abcd.dll"

        process_obj = Process()
        process_obj.image_info = ImageInfo()
        process_obj.image_info.path = file_path_string

        normalize_object_properties(process_obj)

        self.assertEqual(process_obj.image_info.path.value, normalized_file_path_string)

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.objects.win_thread_object import WinThread
from cybox.test.objects import ObjectTestCase


class TestWinThread(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsThreadObjectType"
    klass = WinThread

    _full_dict = {
        'thread_id': 1234,
        'handle': {
                'name': u("Thread Handle"),
                'type': u("Thread"),
                'xsi:type': "WindowsHandleObjectType",
            },
        'running_status': u("Ready"),
        'context': u("My Context"),
        'priority': 15,
        'creation_flags': u("FFF0"),
        'creation_time': "2013-07-31T14:08:10+05:00",
        'start_address': u("00400000"),
        'parameter_address': u("01234567"),
        'security_attributes': u("Some Attributes"),
        'stack_size': 0x1000,
        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.cybox_common as common_binding
from cybox.common import HashList, Integer, String, PositiveInteger


class ByteRun(entities.Entity):
    _binding = common_binding
    _binding_class = common_binding.ByteRunType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'

    offset = fields.TypedField("Offset", Integer)
    byte_order = fields.TypedField("Byte_Order", String)
    file_system_offset = fields.TypedField("File_System_Offset", Integer)
    image_offset = fields.TypedField("Image_Offset", Integer)
    length = fields.TypedField("Length", PositiveInteger)
    hashes = fields.TypedField("Hashes", HashList)
    byte_run_data = fields.TypedField("Byte_Run_Data")


class ByteRuns(entities.EntityList):
    _binding_class = common_binding.ByteRunsType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/common-2'
    byte_run = fields.TypedField("Byte_Run", ByteRun, multiple=True)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields
from mixbox.vendor import six

import cybox.bindings.uri_object as uri_binding
from cybox.common import ObjectProperties, AnyURI


@six.python_2_unicode_compatible
class URI(ObjectProperties):
    _binding = uri_binding
    _binding_class = uri_binding.URIObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/uri-2'
    _XSI_NS = 'URIObj'
    _XSI_TYPE = "URIObjectType"

    TYPE_URL = "URL"
    TYPE_GENERAL = "General URN"
    TYPE_DOMAIN = "Domain Name"

    TYPES = (TYPE_URL, TYPE_GENERAL, TYPE_DOMAIN)

    value = fields.TypedField("Value", AnyURI)
    type_ = fields.TypedField("type_", key_name="type")

    def __init__(self, value=None, type_=None):
        super(URI, self).__init__()
        self.value = value
        self.type_ = type_

    def __str__(self):
        return six.text_type(self.value)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.volume_object as volume_binding
from cybox.common import ObjectProperties, String, DateTime, UnsignedLong, PositiveInteger, UnsignedInteger


class FileSystemFlagList(entities.EntityList):
    _binding = volume_binding
    _binding_class = volume_binding.FileSystemFlagListType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/volume-2"
    file_system_flag = fields.TypedField("File_System_Flag", String, multiple=True)


class Volume(ObjectProperties):
    _binding = volume_binding
    _binding_class = volume_binding.VolumeObjectType
    _namespace = "http://docs.oasis-open.org/cti/ns/cybox/objects/volume-2"
    _XSI_NS = "VolumeObj"
    _XSI_TYPE = "VolumeObjectType"

    is_mounted = fields.TypedField('is_mounted')
    name = fields.TypedField('Name', String)
    device_path = fields.TypedField('Device_Path', String)
    file_system_type = fields.TypedField('File_System_Type', String)
    total_allocation_units = fields.TypedField('Total_Allocation_Units', UnsignedLong)
    sectors_per_allocation_unit = fields.TypedField('Sectors_Per_Allocation_Unit', UnsignedInteger)
    bytes_per_sector = fields.TypedField('Bytes_Per_Sector', PositiveInteger)
    actual_available_allocation_units = fields.TypedField('Actual_Available_Allocation_Units', UnsignedLong)
    creation_time = fields.TypedField('Creation_Time', DateTime)
    file_system_flag_list = fields.TypedField('File_System_Flag_List', FileSystemFlagList)
    serial_number = fields.TypedField('Serial_Number', String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields
from mixbox.vendor import six

import cybox.bindings.address_object as address_binding
from cybox.common import ObjectProperties, String, Integer


@six.python_2_unicode_compatible
class Address(ObjectProperties):
    _binding = address_binding
    _binding_class = _binding.AddressObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/address-2'
    _value_field = 'address_value'
    _XSI_NS = 'AddressObj'
    _XSI_TYPE = 'AddressObjectType'

    CAT_ASN = "asn"
    CAT_ATM = "atm"
    CAT_CIDR = "cidr"
    CAT_EMAIL = "e-mail"
    CAT_MAC = "mac"
    CAT_IPV4 = "ipv4-addr"
    CAT_IPV4_NET = "ipv4-net"
    CAT_IPV4_NETMASK = "ipv4-netmask"
    CAT_IPV6 = "ipv6-addr"
    CAT_IPV6_NET = "ipv6-net"
    CAT_IPV6_NETMASK = "ipv6-netmask"

    address_value = fields.TypedField("Address_Value", String)
    category = fields.TypedField("category")
    is_destination = fields.TypedField("is_destination")
    is_source = fields.TypedField("is_source")
    is_spoofed = fields.TypedField("is_spoofed")
    vlan_name = fields.TypedField("VLAN_Name", String)
    vlan_num = fields.TypedField("VLAN_Num", Integer)

    def __init__(self, address_value=None, category=None):
        super(Address, self).__init__()
        self.address_value = address_value
        self.category = category

    def __str__(self):
        return six.text_type(self.address_value)

    # Shortcuts
    @property
    def condition(self):
        return self.address_value.condition

    @condition.setter
    def condition(self, value):
        self.address_value.condition = value


class EmailAddress(Address):
    """Convenience class for creating email addresses.

    Note that this is not an actual CybOX type."""

    def __init__(self, addr_string=None):
        super(EmailAddress, self).__init__(addr_string, Address.CAT_EMAIL)

    @classmethod
    def istypeof(cls, obj):
        return isinstance(obj, Address) and obj.category == Address.CAT_EMAIL



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from mixbox.vendor.six import u

from cybox.compat import long
from cybox.objects.win_file_object import WinFile, Stream
from cybox.test.common.hash_test import EMPTY_MD5
from cybox.test import EntityTestCase
from cybox.test.objects import ObjectTestCase


class TestStream(EntityTestCase, unittest.TestCase):
    klass = Stream
    _full_dict = {'hashes': [{'type': u("MD5"),
                              'simple_hash_value': EMPTY_MD5}],
                  'name': u("StreamB"),
                  'size_in_bytes': 204}


class TestWinFile(ObjectTestCase, unittest.TestCase):
    object_type = "WindowsFileObjectType"
    klass = WinFile

    _full_dict = {
        # File fields (only a few are included)
        'file_name': u("example.doc"),
        'full_path': u("C:\\Temp\\example.doc"),
        'file_extension': u("doc"),
        'size_in_bytes': long(1024),
        'magic_number': u("D0CF11E0"),

        # WinFile-specific fields
        'filename_accessed_time': "2012-05-12T07:14:02+07:00",
        'filename_created_time': "2012-05-17T09:28:04+07:00",
        'filename_modified_time': "2012-06-12T11:15:12+07:00",
        'drive': u("C:"),
        'security_id': u("S-1-5-21-3623958015-3361044348-30300820-1013"),
        'security_type': u("SidTypeFile"),
        'stream_list': [{'name': u("StreamA")},
                        {'hashes': [{'type': u("MD5"),
                                     'simple_hash_value': EMPTY_MD5}],
                         'name': u("StreamB")}],

        # WinFile-specific implementations of abstract types.
        'file_attributes_list': [u("Hidden"), u("System"), u("Temporary")],
        'permissions': {
            'full_control': False,
            'modify': True,
            'read': True,
            'read_and_execute': False,
            'write': False,
        },

        'xsi:type': object_type,
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.common import Contributor
from cybox.test import EntityTestCase


class TestContributor(EntityTestCase, unittest.TestCase):
    klass = Contributor

    _full_dict = {
        'role': "Lead Tester",
        'name': "Marty McFly",
        'email': "marty@mcfly.com",
        'phone': "(123) 456-7890",
        'organization': "Doc Brown Enterprises(tm)",
        'date': {
            'start_date': "1955-11-05",
            'end_date': "1985-11-05",
        },
        'contribution_location': "Hill Valley, CA",
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from __future__ import absolute_import

from .frequency import Frequency

from .object import (DomainSpecificObjectProperties, Object, RelatedObject,
        Relationship)

from .action_reference import ActionReference
from .associated_object import AssociatedObject, AssociationType
from .action import (Action, ActionAliases, ActionArgument, ActionArguments,
        ActionName, ActionRelationship, ActionRelationships, Actions,
        ActionType, ArgumentName, AssociatedObjects)

from .event import Event, EventType
from .pattern_fidelity import (PatternFidelity, ObfuscationTechniques,
                               ObfuscationTechnique)
from .observable import Observable, Observables, ObservableComposition



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from cybox.objects.network_subnet_object import NetworkSubnet

from cybox.test import EntityTestCase, round_trip
from cybox.test.objects import ObjectTestCase


class TestNetworkSubnet(ObjectTestCase, unittest.TestCase):
    object_type = "NetworkSubnetObjectType"
    klass = NetworkSubnet

    _full_dict = {
        'name': "A test subnet",
        'description': "Test subnet",
        'number_of_ip_addresses': 2,
        #'routes': [],
        'xsi:type': object_type,
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import datetime
import logging
import unittest

from mixbox.vendor.six import u

from cybox.common import String, DateTime
from cybox.core import Observables
from cybox.objects.address_object import Address, EmailAddress
from cybox.objects.email_message_object import (AttachmentReference,
        Attachments, EmailHeader, EmailMessage, EmailRecipients, LinkReference,
        Links, ReceivedLine, ReceivedLineList)
from cybox.objects.uri_object import URI
import cybox.test
from cybox.test import EntityTestCase
from cybox.test.objects import ObjectTestCase

logger = logging.getLogger(__name__)


class TestLinks(EntityTestCase, unittest.TestCase):
    klass = LinkReference
    _full_dict = {'object_reference': "example:URI-C2"}

    def test_round_trip_list(self):
        l = Links()
        l.append("example:URI-watchlist1")
        l.append("example:URI-watchlist2")
        l2 = cybox.test.round_trip(l, list_=True)
        self.assertEqual(l.to_list(), l2.to_list())


class TestAttachments(EntityTestCase, unittest.TestCase):
    klass = AttachmentReference
    _full_dict = {'object_reference': "abc-123"}

    def test_round_trip_list(self):
        a = Attachments()
        a.append("example:File-A1")
        a.append("example:File-A3")
        a2 = cybox.test.round_trip(a, list_=True)
        self.assertEqual(a.to_list(), a2.to_list())


class TestReceivedLine(EntityTestCase, unittest.TestCase):
    klass = ReceivedLine

    _full_dict = {
        'from': "sending.mail.server",
        'by': "receiving.mail.server",
        'via': "TCP",
        'with': "ESMTP",
        'for': "recipient@example.com",
        'id': "test.id@test.local",
        'timestamp': "2013-04-29T13:00:00-05:00"
    }

    def test_round_trip_partial(self):
        rline_dict = {
                        'from': "bad.mail.server",
                        'for': "victim@example.com",
                     }
        rline_dict2 = cybox.test.round_trip_dict(ReceivedLine, rline_dict)
        self.assertEqual(rline_dict, rline_dict2)

    def test_empty_round_trip(self):
        rline_dict = {}
        rline_dict2 = cybox.test.round_trip_dict(ReceivedLine, rline_dict)
        self.assertEqual(rline_dict, rline_dict2)


class TestReceivedLineList(unittest.TestCase):

    def test_round_trip(self):
        rline_list = [{'from': "a", 'by': "b", 'for': 'c'},
                      {'from': "d", 'by': "e", 'for': 'f'}]
        rline_list2 = cybox.test.round_trip_list(ReceivedLineList, rline_list)
        self.assertEqual(rline_list, rline_list2)


class TestEmailRecipients(unittest.TestCase):
    def setUp(self):
        self.email1 = "victim1@target.com"
        self.email2 = "victim2@target.com"

    # Five different ways to build an EmailRecipients list

    def test_list(self):
        recips = EmailRecipients(Address(self.email1, Address.CAT_EMAIL),
                                 Address(self.email2, Address.CAT_EMAIL))
        self._compare(recips)

    def test_list2(self):
        recips = EmailRecipients(EmailAddress(self.email1),
                                 EmailAddress(self.email2))
        self._compare(recips)

    def test_list3(self):
        recips = EmailRecipients(self.email1, self.email2)
        self._compare(recips)

    def test_list4(self):
        recips = EmailRecipients()
        recips.append(self.email1)
        recips.append(self.email2)
        self._compare(recips)

    def test_list5(self):
        recips = EmailRecipients()
        recips.append(EmailAddress(self.email1))
        recips.append(EmailAddress(self.email2))
        self._compare(recips)

    def _compare(self, recips):
        recips2 = cybox.test.round_trip(recips, list_=True)
        self.assertEqual(2, len(recips2))

        recips_list = recips2.to_list()
        self.assertEqual(recips_list[0]['category'], Address.CAT_EMAIL)
        self.assertEqual(recips_list[0]['address_value'], self.email1)
        self.assertEqual(recips_list[1]['category'], Address.CAT_EMAIL)
        self.assertEqual(recips_list[1]['address_value'], self.email2)

    def test_invalid_recip_type(self):
        ipv4 = Address("1.2.3.4", Address.CAT_IPV4)
        generic_address = Address("aaaa")
        for a in [{1: 'a'}, 1, True, [1], ipv4, generic_address]:
            self.assertRaises(TypeError, EmailRecipients, a)


class TestEmailHeader(EntityTestCase, unittest.TestCase):
    klass = EmailHeader

    _full_dict = {
        'received_lines': [{'from': "a", 'by': "b"}],
        'to': [{'address_value': "victim@example.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE}],
        'cc': [{'address_value': "victim2@example.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE}],
        'bcc': [{'address_value': "victim3@example.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE}],
        'from': {'address_value': "badguy@attacker.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE},
        'subject': "This is not a malicious email",
        'in_reply_to': "<123456@mail.example.com>",
        'date': "2010-12-10T14:15:30+02:00",
        'message_id': "<abcdef@mail.attacker.com>",
        'sender': {'address_value': "attacker2@example.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE},
        'reply_to': {'address_value': "greyhat@attacker.com",
                'category': Address.CAT_EMAIL,
                'xsi:type': Address._XSI_TYPE},
        'errors_to': "/dev/null",
        'boundary': "----MIME_BOUNDARY------",
        'content_type': "mime/multi-part",
        'mime_version': "1.0",
        'precedence': "High",
        'user_agent': "Outlook_Express1.0",
        'x_mailer': "Outlook Express",
        'x_originating_ip': {'address_value': "1.2.3.4",
                            'category': Address.CAT_IPV4,
                            'xsi:type': Address._XSI_TYPE},
        'x_priority': 3,
    }

    def test_datetime(self):
        header = EmailHeader.from_dict(self._full_dict)
        self.assertEqual(DateTime, type(header.date))

    def test_creation(self):
        header = EmailHeader()

        self.assertEqual(None, header.subject)
        header.subject = "Howdy"
        self.assertEqual(String, type(header.subject))

        self.assertEqual(None, header.date)
        header.date = "2010-12-10T14:15:30+02:00"
        self.assertEqual(DateTime, type(header.date))
        self.assertEqual(datetime.datetime, type(header.date.value))

        self.assertEqual(None, header.message_id)
        header.message_id = "<1bc5nkmvakjn45mn@example.com>"
        self.assertEqual(String, type(header.message_id))

    def test_subject_TypedField(self):
        h = EmailHeader()

        # Set using actual object
        h.subject = String("Howdy")
        self.assertEqual(String, type(h.subject))

        # Set using implied cast
        h.subject = "Howdy"
        self.assertEqual(String, type(h.subject))

        s = "http://badsubject.com"
        bad_object = URI(s)
        self.assertRaises(ValueError, setattr, h, 'subject', bad_object)

    def test_sender_TypedField(self):
        h = EmailHeader()
        a = "bob@example.com"

        # Set using actual object
        h.sender = Address(a, Address.CAT_EMAIL)
        # In this case, the type is Address, not EmailAddress, but we don't
        # care since EmailAddress.istypeof(h.sender) is True
        self.assertNotEqual(EmailAddress, type(h.sender))
        self.assertEqual(Address, type(h.sender))
        self.assertTrue(EmailAddress.istypeof(h.sender))

        # Set using "aliased" object
        h.sender = EmailAddress(a)
        # In this case it is actually an EmailAddress, not just an Address
        # (but isinstance returns True)
        self.assertEqual(EmailAddress, type(h.sender))
        self.assertNotEqual(Address, type(h.sender))
        self.assertTrue(isinstance(h.sender, Address))
        self.assertTrue(EmailAddress.istypeof(h.sender))

        h.sender = a
        self.assertTrue(EmailAddress.istypeof(h.sender))

        bad_object = 42
        self.assertRaises(ValueError, setattr, h, 'sender', bad_object)

        # Just because it's an Address isn't good enough. It needs to have
        # the right category.
        ipv4_object = Address(a, Address.CAT_IPV4)
        self.assertRaises(ValueError, setattr, h, 'sender', ipv4_object)


class TestEmailMessage(ObjectTestCase, unittest.TestCase):
    object_type = "EmailMessageObjectType"
    klass = EmailMessage

    _full_dict = {
        #TODO: populate
        'raw_body': u("This has some unicode \ufffd characters"),
        'xsi:type': object_type,
    }

    def test_roundtrip_basic(self):
        msg_dict = {'header': {
                                'from': "sender@domain.org",
                                'to': ["recip@victim.org"],
                                'subject': "Howdy!"
                               },
                     'raw_body': "This is a test. This is only a test.",
                    }
        msg_obj = EmailMessage.object_from_dict(msg_dict)
        msg_dict2 = EmailMessage.dict_from_object(msg_obj)

        # Don't want to compare dictionaries directly since email addresses
        # will have been expanded to full dictionaries with "address_value"
        # and "category"
        self.assertEqual(msg_dict2['header']['from']['address_value'],
                         msg_dict['header']['from'])
        self.assertEqual(msg_dict2['header']['to'][0]['address_value'],
                         msg_dict['header']['to'][0])
        self.assertEqual(msg_dict2['header']['subject'],
                         msg_dict['header']['subject'])
        self.assertEqual(msg_dict2['raw_body'],
                         msg_dict['raw_body'])
        self.assertEqual(msg_dict2['xsi:type'], EmailMessage._XSI_TYPE)

        # Make sure extra keys didn't sneak into the output.
        self.assertFalse('attachments' in msg_dict2)
        self.assertFalse('optional_header' in msg_dict2)
        self.assertFalse('email_server' in msg_dict2)
        self.assertFalse('raw_header' in msg_dict2)
        self.assertFalse('cc' in msg_dict2['header'])
        self.assertFalse('bcc' in msg_dict2['header'])
        self.assertFalse('in_reply_to' in msg_dict2['header'])
        self.assertFalse('date' in msg_dict2['header'])
        self.assertFalse('message_id' in msg_dict2['header'])
        self.assertFalse('sender' in msg_dict2['header'])
        self.assertFalse('reply_to' in msg_dict2['header'])
        self.assertFalse('errors_to' in msg_dict2['header'])

    def test_from_date(self):
        date_str = "Thu, 14 Feb 2013 11:28:42 -0500"
        isoformat = "2013-02-14T11:28:42-05:00"

        d = {'header': {'date': date_str}}
        msg = EmailMessage.from_dict(d)
        self.assertEqual(msg.date.serialized_value, isoformat)
        self.assertEqual(msg.date.to_obj().valueOf_, isoformat)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import sys

from mixbox.binding_utils import *
from . import cybox_common
from . import win_handle_object


class PageProtectionAttributeType(cybox_common.BaseObjectPropertyType):
    """The PageProtectionValueType specifies the optional Windows file
    mapping page protection attribute types (i.e. SEC_) via a union
    of the PageProtectionAttributeEnum type and the atomic xs:string
    type. Its base type is the CybOX Core cybox_common.BaseObjectPropertyType,
    for permitting complex (i.e. regular-expression based)
    specifications."""

    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(PageProtectionAttributeType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if PageProtectionAttributeType.subclass:
            return PageProtectionAttributeType.subclass(*args_, **kwargs_)
        else:
            return PageProtectionAttributeType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(PageProtectionAttributeType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinFilemappingObj:', name_='PageProtectionAttributeType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='PageProtectionAttributeType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinFilemappingObj:', name_='PageProtectionAttributeType'):
        super(PageProtectionAttributeType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='PageProtectionAttributeType')
    def exportChildren(self, lwrite, level, namespace_='WinFilemappingObj:', name_='PageProtectionAttributeType', fromsubclass_=False, pretty_print=True):
        super(PageProtectionAttributeType, self).exportChildren(lwrite, level, 'WinFilemappingObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(PageProtectionAttributeType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class PageProtectionAttributeType

class PageProtectionValueType(cybox_common.BaseObjectPropertyType):
    """The PageProtectionValueType specifies Windows file mapping page
    protection value types (i.e. PAGE_) via a union of the
    PageProtectionValueEnum type and the atomic xs:string type. Its
    base type is the CybOX Core cybox_common.BaseObjectPropertyType, for
    permitting complex (i.e. regular-expression based)
    specifications."""
    subclass = None
    superclass = cybox_common.BaseObjectPropertyType
    def __init__(self, obfuscation_algorithm_ref=None, refanging_transform_type=None, has_changed=None, delimiter='##comma##', pattern_type=None, datatype='string', refanging_transform=None, is_case_sensitive=True, bit_mask=None, appears_random=None, observed_encoding=None, defanging_algorithm_ref=None, is_obfuscated=None, regex_syntax=None, apply_condition='ANY', trend=None, idref=None, is_defanged=None, id=None, condition=None, valueOf_=None):
        super(PageProtectionValueType, self).__init__(obfuscation_algorithm_ref, refanging_transform_type, has_changed, delimiter, pattern_type, datatype, refanging_transform, is_case_sensitive, bit_mask, appears_random, observed_encoding, defanging_algorithm_ref, is_obfuscated, regex_syntax, apply_condition, trend, idref, is_defanged, id, condition, valueOf_)
        self.valueOf_ = valueOf_
    def factory(*args_, **kwargs_):
        if PageProtectionValueType.subclass:
            return PageProtectionValueType.subclass(*args_, **kwargs_)
        else:
            return PageProtectionValueType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_valueOf_(self): return self.valueOf_
    def set_valueOf_(self, valueOf_): self.valueOf_ = valueOf_
    def hasContent_(self):
        if (
            self.valueOf_ or
            super(PageProtectionValueType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinFilemappingObj:', name_='PageProtectionValueType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='PageProtectionValueType')
        if self.hasContent_():
            lwrite('>')
            lwrite(quote_xml(self.valueOf_))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinFilemappingObj:', name_='PageProtectionValueType'):
        super(PageProtectionValueType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='PageProtectionValueType')
    def exportChildren(self, lwrite, level, namespace_='WinFilemappingObj:', name_='PageProtectionValueType', fromsubclass_=False, pretty_print=True):
        super(PageProtectionValueType, self).exportChildren(lwrite, level, 'WinFilemappingObj:', name_, True, pretty_print=pretty_print)
        pass
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        self.valueOf_ = get_all_text_(node)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(PageProtectionValueType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        pass
# end class PageProtectionValueType

class WindowsFilemappingObjectType(cybox_common.ObjectPropertiesType):
    """The WindowsFilemappingObjectType type is intended to characterize
    Windows file mapping objects."""

    subclass = None
    superclass = cybox_common.ObjectPropertiesType
    def __init__(self, object_reference=None, Custom_Properties=None, xsi_type=None, Name=None, File_Handle=None, Handle=None, Page_Protection_Value=None, Page_Protection_Attribute=None, Maximum_Size=None, Actual_Size=None, Security_Attributes=None):
        super(WindowsFilemappingObjectType, self).__init__(object_reference, Custom_Properties, xsi_type)
        self.Name = Name
        self.File_Handle = File_Handle
        self.Handle = Handle
        self.Page_Protection_Value = Page_Protection_Value
        if Page_Protection_Attribute is None:
            self.Page_Protection_Attribute = []
        else:
            self.Page_Protection_Attribute = Page_Protection_Attribute
        self.Maximum_Size = Maximum_Size
        self.Actual_Size = Actual_Size
        self.Security_Attributes = Security_Attributes
    def factory(*args_, **kwargs_):
        if WindowsFilemappingObjectType.subclass:
            return WindowsFilemappingObjectType.subclass(*args_, **kwargs_)
        else:
            return WindowsFilemappingObjectType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_Name(self): return self.Name
    def set_Name(self, Name): self.Name = Name
    def validate_StringObjectPropertyType(self, value):
        # Validate type cybox_common.StringObjectPropertyType, a restriction on None.
        pass
    def get_File_Handle(self): return self.File_Handle
    def set_File_Handle(self, File_Handle): self.File_Handle = File_Handle
    def get_Handle(self): return self.Handle
    def set_Handle(self, Handle): self.Handle = Handle
    def get_Page_Protection_Value(self): return self.Page_Protection_Value
    def set_Page_Protection_Value(self, Page_Protection_Value): self.Page_Protection_Value = Page_Protection_Value
    def validate_PageProtectionValueType(self, value):
        # Validate type PageProtectionValueType, a restriction on None.
        pass
    def get_Page_Protection_Attribute(self): return self.Page_Protection_Attribute
    def set_Page_Protection_Attribute(self, Page_Protection_Attribute): self.Page_Protection_Attribute = Page_Protection_Attribute
    def add_Page_Protection_Attribute(self, value): self.Page_Protection_Attribute.append(value)
    def insert_Page_Protection_Attribute(self, index, value): self.Page_Protection_Attribute[index] = value
    def validate_PageProtectionAttributeType(self, value):
        # Validate type PageProtectionAttributeType, a restriction on None.
        pass
    def get_Maximum_Size(self): return self.Maximum_Size
    def set_Maximum_Size(self, Maximum_Size): self.Maximum_Size = Maximum_Size
    def validate_UnsignedLongObjectPropertyType(self, value):
        # Validate type cybox_common.UnsignedLongObjectPropertyType, a restriction on None.
        pass
    def get_Actual_Size(self): return self.Actual_Size
    def set_Actual_Size(self, Actual_Size): self.Actual_Size = Actual_Size
    def get_Security_Attributes(self): return self.Security_Attributes
    def set_Security_Attributes(self, Security_Attributes): self.Security_Attributes = Security_Attributes
    def hasContent_(self):
        if (
            self.Name is not None or
            self.File_Handle is not None or
            self.Handle is not None or
            self.Page_Protection_Value is not None or
            self.Page_Protection_Attribute or
            self.Maximum_Size is not None or
            self.Actual_Size is not None or
            self.Security_Attributes is not None or
            super(WindowsFilemappingObjectType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, namespace_='WinFilemappingObj:', name_='WindowsFilemappingObjectType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s%s%s' % (namespace_, name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsFilemappingObjectType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, namespace_, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s%s>%s' % (namespace_, name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='WinFilemappingObj:', name_='WindowsFilemappingObjectType'):
        super(WindowsFilemappingObjectType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='WindowsFilemappingObjectType')
    def exportChildren(self, lwrite, level, namespace_='WinFilemappingObj:', name_='WindowsFilemappingObjectType', fromsubclass_=False, pretty_print=True):
        super(WindowsFilemappingObjectType, self).exportChildren(lwrite, level, 'WinFilemappingObj:', name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.Name is not None:
            self.Name.export(lwrite, level, 'WinFilemappingObj:', name_='Name', pretty_print=pretty_print)
        if self.File_Handle is not None:
            self.File_Handle.export(lwrite, level, 'WinFilemappingObj:', name_='File_Handle', pretty_print=pretty_print)
        if self.Handle is not None:
            self.Handle.export(lwrite, level, 'WinFilemappingObj:', name_='Handle', pretty_print=pretty_print)
        if self.Page_Protection_Value is not None:
            self.Page_Protection_Value.export(lwrite, level, 'WinFilemappingObj:', name_='Page_Protection_Value', pretty_print=pretty_print)
        for Page_Protection_Attribute_ in self.Page_Protection_Attribute:
            Page_Protection_Attribute_.export(lwrite, level, 'WinFilemappingObj:', name_='Page_Protection_Attribute', pretty_print=pretty_print)
        if self.Maximum_Size is not None:
            self.Maximum_Size.export(lwrite, level, 'WinFilemappingObj:', name_='Maximum_Size', pretty_print=pretty_print)
        if self.Actual_Size is not None:
            self.Actual_Size.export(lwrite, level, 'WinFilemappingObj:', name_='Actual_Size', pretty_print=pretty_print)
        if self.Security_Attributes is not None:
            self.Security_Attributes.export(lwrite, level, 'WinFilemappingObj:', name_='Security_Attributes', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(WindowsFilemappingObjectType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'Name':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Name(obj_)
        elif nodeName_ == 'File_Handle':
            obj_ = win_handle_object.WindowsHandleObjectType.factory()
            obj_.build(child_)
            self.set_File_Handle(obj_)
        elif nodeName_ == 'Handle':
            obj_ = win_handle_object.WindowsHandleObjectType.factory()
            obj_.build(child_)
            self.set_Handle(obj_)
        elif nodeName_ == 'Page_Protection_Value':
            obj_ = PageProtectionValueType.factory()
            obj_.build(child_)
            self.set_Page_Protection_Value(obj_)
        elif nodeName_ == 'Page_Protection_Attribute':
            obj_ = PageProtectionAttributeType.factory()
            obj_.build(child_)
            self.Page_Protection_Attribute.append(obj_)
        elif nodeName_ == 'Maximum_Size':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Maximum_Size(obj_)
        elif nodeName_ == 'Actual_Size':
            obj_ = cybox_common.UnsignedLongObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Actual_Size(obj_)
        elif nodeName_ == 'Security_Attributes':
            obj_ = cybox_common.StringObjectPropertyType.factory()
            obj_.build(child_)
            self.set_Security_Attributes(obj_)
        super(WindowsFilemappingObjectType, self).buildChildren(child_, node, nodeName_, True)
# end class WindowsFilemappingObjectType

GDSClassesMapping = {
    'Build_Utility': cybox_common.BuildUtilityType,
    'Byte_Order': cybox_common.EndiannessType,
    'Errors': cybox_common.ErrorsType,
    'Time': cybox_common.TimeType,
    'Certificate_Issuer': cybox_common.StringObjectPropertyType,
    'Metadata': cybox_common.MetadataType,
    'Hash': cybox_common.HashType,
    'Information_Source_Type': cybox_common.ControlledVocabularyStringType,
    'Internal_Strings': cybox_common.InternalStringsType,
    'SubDatum': cybox_common.MetadataType,
    'Segment_Hash': cybox_common.HashValueType,
    'Digital_Signature': cybox_common.DigitalSignatureInfoType,
    'Code_Snippets': cybox_common.CodeSnippetsType,
    'Value': cybox_common.StringObjectPropertyType,
    'Length': cybox_common.IntegerObjectPropertyType,
    'Produced_Time': cybox_common.DateTimeWithPrecisionType,
    'Actual_Size': cybox_common.UnsignedLongObjectPropertyType,
    'Reference': cybox_common.ToolReferenceType,
    'Encoding': cybox_common.ControlledVocabularyStringType,
    'Internationalization_Settings': cybox_common.InternationalizationSettingsType,
    'Tool_Configuration': cybox_common.ToolConfigurationType,
    'Security_Attributes': cybox_common.StringObjectPropertyType,
    'Object_Address': cybox_common.UnsignedLongObjectPropertyType,
    'Compiler': cybox_common.CompilerType,
    'Start_Date': cybox_common.DateWithPrecisionType,
    'Functions': cybox_common.FunctionsType,
    'String_Value': cybox_common.StringObjectPropertyType,
    'Pointer_Count': cybox_common.UnsignedLongObjectPropertyType,
    'Build_Utility_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Compiler_Informal_Description': cybox_common.CompilerInformalDescriptionType,
    'Start_Time': cybox_common.DateTimeWithPrecisionType,
    'System': cybox_common.ObjectPropertiesType,
    'Platform': cybox_common.PlatformSpecificationType,
    'Usage_Context_Assumptions': cybox_common.UsageContextAssumptionsType,
    'Type': win_handle_object.HandleType,
    'Compilers': cybox_common.CompilersType,
    'Tool_Type': cybox_common.ControlledVocabularyStringType,
    'String': cybox_common.ExtractedStringType,
    'Custom_Properties': cybox_common.CustomPropertiesType,
    'Build_Information': cybox_common.BuildInformationType,
    'Tool_Hashes': cybox_common.HashListType,
    'Maximum_Size': cybox_common.UnsignedLongObjectPropertyType,
    'Observable_Location': cybox_common.LocationType,
    'Error_Instances': cybox_common.ErrorInstancesType,
    'End_Date': cybox_common.DateWithPrecisionType,
    'Data_Segment': cybox_common.StringObjectPropertyType,
    'Certificate_Subject': cybox_common.StringObjectPropertyType,
    'Compensation_Model': cybox_common.CompensationModelType,
    'Property': cybox_common.PropertyType,
    'Strings': cybox_common.ExtractedStringsType,
    'File_Handle': win_handle_object.WindowsHandleObjectType,
    'Contributors': cybox_common.PersonnelType,
    'User_Account_Info': cybox_common.ObjectPropertiesType,
    'Configuration_Settings': cybox_common.ConfigurationSettingsType,
    'Compiler_Platform_Specification': cybox_common.PlatformSpecificationType,
    'Byte_String_Value': cybox_common.HexBinaryObjectPropertyType,
    'Received_Time': cybox_common.DateTimeWithPrecisionType,
    'Instance': cybox_common.ObjectPropertiesType,
    'Import': cybox_common.StringObjectPropertyType,
    'Access_Mask': cybox_common.UnsignedLongObjectPropertyType,
    'Identifier': cybox_common.PlatformIdentifierType,
    'Tool_Specific_Data': cybox_common.ToolSpecificDataType,
    'Execution_Environment': cybox_common.ExecutionEnvironmentType,
    'Search_Distance': cybox_common.IntegerObjectPropertyType,
    'Dependencies': cybox_common.DependenciesType,
    'Segment_Count': cybox_common.IntegerObjectPropertyType,
    'Offset': cybox_common.IntegerObjectPropertyType,
    'Date': cybox_common.DateRangeType,
    'Hashes': cybox_common.HashListType,
    'Segments': cybox_common.HashSegmentsType,
    'Language': cybox_common.StringObjectPropertyType,
    'Usage_Context_Assumption': cybox_common.StructuredTextType,
    'Block_Hash': cybox_common.FuzzyHashBlockType,
    'Dependency': cybox_common.DependencyType,
    'Error': cybox_common.ErrorType,
    'ID': cybox_common.UnsignedIntegerObjectPropertyType,
    'Trigger_Point': cybox_common.HexBinaryObjectPropertyType,
    'Environment_Variable': cybox_common.EnvironmentVariableType,
    'Byte_Run': cybox_common.ByteRunType,
    'File_System_Offset': cybox_common.IntegerObjectPropertyType,
    'Image_Offset': cybox_common.IntegerObjectPropertyType,
    'Imports': cybox_common.ImportsType,
    'Library': cybox_common.LibraryType,
    'References': cybox_common.ToolReferencesType,
    'Windows_Handle': win_handle_object.WindowsHandleObjectType,
    'Compilation_Date': cybox_common.DateTimeWithPrecisionType,
    'Block_Hash_Value': cybox_common.HashValueType,
    'Fuzzy_Hash_Structure': cybox_common.FuzzyHashStructureType,
    'Configuration_Setting': cybox_common.ConfigurationSettingType,
    'Observation_Location': cybox_common.LocationType,
    'Libraries': cybox_common.LibrariesType,
    'Function': cybox_common.StringObjectPropertyType,
    'Handle': win_handle_object.WindowsHandleObjectType,
    'Description': cybox_common.StructuredTextType,
    'Code_Snippet': cybox_common.ObjectPropertiesType,
    'Build_Configuration': cybox_common.BuildConfigurationType,
    'Address': cybox_common.HexBinaryObjectPropertyType,
    'Search_Within': cybox_common.IntegerObjectPropertyType,
    'Segment': cybox_common.HashSegmentType,
    'English_Translation': cybox_common.StringObjectPropertyType,
    'Name': cybox_common.StringObjectPropertyType,
    'Signature_Description': cybox_common.StringObjectPropertyType,
    'Block_Size': cybox_common.IntegerObjectPropertyType,
    'Simple_Hash_Value': cybox_common.SimpleHashValueType,
    'Fuzzy_Hash_Value': cybox_common.FuzzyHashValueType,
    'Data_Size': cybox_common.DataSizeType,
    'Dependency_Description': cybox_common.StructuredTextType,
    'End_Time': cybox_common.DateTimeWithPrecisionType,
    'Contributor': cybox_common.ContributorType,
    'Tools': cybox_common.ToolsInformationType,
    'Tool': cybox_common.ToolInformationType,
}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Filemapping'
        rootClass = WindowsFilemappingObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_=rootTag,
        namespacedef_='',
        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Filemapping'
        rootClass = WindowsFilemappingObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'Windows_Filemapping'
        rootClass = WindowsFilemappingObjectType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout.write, 0, name_="Windows_Filemapping",
        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "WindowsFilemappingObjectType",
    "PageProtectionValueType",
    "PageProtectionAttributeType"
    ]



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import cybox.bindings.win_semaphore_object as semaphore_binding
from cybox.common import String
from cybox.objects.win_handle_object import WinHandle
from cybox.objects.semaphore_object import Semaphore


class WinSemaphore(Semaphore):
    _binding = semaphore_binding
    _binding_class = semaphore_binding.WindowsSemaphoreObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-semaphore-2'
    _XSI_NS = "WinSemaphoreObj"
    _XSI_TYPE = "WindowsSemaphoreObjectType"

    handle = fields.TypedField("Handle", WinHandle)
    security_attributes = fields.TypedField("Security_Attributes", String)



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import entities
from mixbox import fields

import cybox.bindings.win_computer_account_object as account_binding
from cybox.common import HexBinary, String, UnsignedLong
from cybox.objects.account_object import Account
from cybox.objects.port_object import Port


class FullyQualifiedName(entities.Entity):
    _binding = account_binding
    _binding_class = account_binding.FullyQualifiedNameType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-computer-account-2'

    netbeui_name = fields.TypedField("NetBEUI_Name", String)
    full_name = fields.TypedField("Full_Name", String)


class KerberosService(entities.Entity):
    _binding = account_binding
    _binding_class = account_binding.KerberosServiceType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-computer-account-2'

    computer = fields.TypedField("Computer", String)
    name = fields.TypedField("Name", String)
    port = fields.TypedField("Port", Port)
    user = fields.TypedField("User", String)


class KerberosDelegation(entities.Entity):
    _binding = account_binding
    _binding_class = account_binding.KerberosDelegationType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-computer-account-2'

    bitmask = fields.TypedField("Bitmask", HexBinary)
    service = fields.TypedField("Service", KerberosService)


class Kerberos(entities.Entity):
    _binding = account_binding
    _binding_class = account_binding.KerberosType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-computer-account-2'

    delegation = fields.TypedField("Delegation", KerberosDelegation)
    ticket = fields.TypedField("Ticket", UnsignedLong)


class WinComputerAccount(Account):
    _binding = account_binding
    _binding_class = account_binding.WindowsComputerAccountObjectType
    _namespace = 'http://docs.oasis-open.org/cti/ns/cybox/objects/win-computer-account-2'
    _XSI_NS = "WinComputerAccountObj"
    _XSI_TYPE = "WindowsComputerAccountObjectType"

    fully_qualified_name = fields.TypedField("Fully_Qualified_Name",
                                            FullyQualifiedName)
    kerberos = fields.TypedField("Kerberos", Kerberos)
    security_id = fields.TypedField("Security_ID", String)
    # TODO: implement common.SIDType
    # security_type = fields.TypedField("Security_Type", SID)
    type_ = fields.TypedField("Type", String)
